{{
uOS v0.3.1
Thomas M. Doylend

This can barely be qualified as uOS.

It provides one 8KB process, EEPROM, serial, and a PS/2 keyboard.
Nothing more.

This distro will only run on the HYDRA. You will need to
change the keyboard driver and the crystal mode to make it
work on another system.

Use the PST, NOT the EST.

Don't forget to format your EEPROM before you use this!!
}}

con
  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000

var

  long sdspi_cntrl[2]

  long pram[2048]

  long slot[17*4] '17 slots * 4 longs per slot.

  long stack[256] '1KB for manager stack.

obj

  dsp : "FullDuplexSerial"
  kbd : "keyboard_iso_010.spin"
  eeprom : "NS_eeprom_drv_011"

pub main | c
  kbd.start(3)
  dsp.start(31,30,0,19_200)
  eeprom.start(28,29,10)
  waitcnt(clkfreq+cnt) 'Wait while everything starts up...
  cognew(manager,@stack)
  dsp.tx(0)
  dsp.tx(1)

  eeprom.Read(@pram,$8000,4096) 'Load BIOS
  repeat until eeprom.isDone
  'repeat c from 0 to 1024
  '  dsp.hex(pram[c],8)
  '  dsp.tx($0D)
    
  Interpreter

pub Interpreter | PC, instr, s_literal, src, dest, src_value, dest_value
  pram[0] := 0         'Program's Storage Device (EEPROM).
  pram[1] := 0         'Pointer to first sector of program.
  pram[2] := -1        'No parent dir.
  pram[3] := -1        'Currently, no ASMDAT.
  pram[4] := 2048      '2048 longs (8KB) PRAM.
  pram[5] := $00_00_0000 'CPUID = 0; PID = 0

  PC := 8

  repeat while true
    instr := pram[PC]
    s_literal := instr >> 31
    src := instr & $FFF
    dest := (instr >> 12) & $FFF
    'dsp.hex(instr,8)
    'dsp.tx($0D)
    instr := (instr>>24) & $7F
    if (s_literal == 1)
      src_value := src
    else
      src_value := pram[src]
    dest_value := pram[dest]
    'dsp.str(string("Got instruction at PC="))
    'dsp.dec(PC)
    'dsp.tx($0D)
    case instr
      $00:              'End self
         'sdspi.readEEPROM(sdspi#bootAddr,@pram,4096) 'reload BIOS
         PC := 8
         repeat while true
           PC := 8
      $01: pram[dest] := src_value               'MOV
      $02: pram[dest] := dest_value+src_value    'ADD
      $03: pram[dest] := dest_value-src_value    'SUB
      $08: pram[dest] := dest_value <- src_value 'ROL
      $09: pram[dest] := dest_value -> src_value 'ROR
      $0A: pram[dest] := dest_value << src_value 'SHL
      $0B: pram[dest] := dest_value >> src_value 'SHR
      $0C: pram[dest] := dest_value &  src_value 'AND
      $0D: pram[dest] := dest_value |  src_value 'OR
      $0E: pram[dest] := dest_value ^  src_value 'XOR
      $0F: pram[dest] := (!src_value)            'NOT
      $10: pram[dest] := (-src_value)            'NEG
      $11: pram[dest] := (||src_value)           'ABS
      $12: pram[dest] := 0                       'RCL is not implemented here because there is no SPIN operator for it.
      $14: pram[dest] := dest_value >< src_value 'REV
      $15: pram[dest] := dest_value & (!src_value) 'ANDN
      $18: if (dest_value <> src_value)
             PC++
      $19: if (dest_value == src_value)
             PC++
      $1A: if (dest_value <= src_value)
             PC++
      $1B: if (dest_value >= src_value)
             PC++
      $20:
        'dsp.tx("#")
        'dsp.dec(src_value)
        'dsp.tx($0D)
        PC := src_value - 1 'Because it will be incremented afterwards.
      $21:
        pram[dest] := PC + 1
        PC := src_value - 1
      $22:
        {dsp.tx("#")
        dsp.dec(src_value)
        dsp.tx(":")
        dsp.dec(dest)
        dsp.tx(":")
        dsp.dec(dest_value)
        dsp.tx($0D)}
        slot[src_value] := dest_value
      $23: pram[dest] := slot[src_value]
      $24: pram[src_value] := dest_value
      $25: pram[dest] := pram[src_value]
    PC++  
      
         

pub manager |dataddr, icntr
  slot[(1*4)+0] := -1
  slot[(16*4)+3] := -1
  repeat while true
    'dsp.str(string("!"))
    if (slot[(0*4)+0] <> 0)
      if (slot[(0*4)+0] == 8)
        slot[(0*4)+2] := long[@os_parms][slot[(0*4)+1]]
      else
        slot[(0*4)+2] := -1 
      slot[(0*4)+0] := 0

    
    if (slot[(2*4)+0] == 0)
      if (slot[(2*4)+1]==0)
        if (kbd.gotkey)
          slot[(2*4)+2] := kbd.getkey
        else
          slot[(2*4)+2] := 0
      else
        slot[(2*4)+2] := kbd.keystate(slot[(2*4)+1])
      slot[(2*4)+0] := -1
    slot[(3*4)+1] := -1  'Lock manager succeeds automatically
    if (slot[(8*4)+0] == 1)
      eeprom.Read(@pram+(4*slot[(8*4)+2]),$8000+(4096*slot[(8*4)+1]),4096)
      repeat until eeprom.isDone
      slot[(8*4)+0] := 0
    if (slot[(8*4)+0] == 2)
      eeprom.Write(@pram+(4*slot[(8*4)+2]),$8000+(4096*slot[(8*4)+1]),4096)
      repeat until eeprom.isDone
      slot[(8*4)+0] := 0
    slot[(8*4)+0] := 0
    if (slot[(16*4)+3] <> -1)
      'dsp.str(string("!",$0D))
      if (slot[(16*4)+2] > 0)
        dataddr := @pram + (4*slot[(16*4)+1])
        repeat slot[(16*4)+2]
          dsp.tx(byte[dataddr])
          dataddr++
        slot[(16*4)+2] := 0
      else
        slot[(16*4)+3] := -1
    


dat

os_parms
long 1                  'Machine type 1: minimal implementation
long $00_01_0001        'Machine version
long $00_03_0001        'OS version
long 0                  'No XRAM
long %0000000           'Flags
long 1                  'Graphics Device 0 (PST)
long 0                  'Graphics Device 1 (none)
    
    
  